#pragma once

#include <C_P1ImageCommonTypes.h>
#include <C_P1ImageCommonLinkage.h>

#include <vector>

namespace P1
{
namespace ImageSdk
{
    /** @brief Declare data type of each histogram channel */
    enum class EXPORT_API HistogramType
    {
        None,
        RGB,
        RGB_L, // RGB + luminosity
        Bayer, // RGGB
    };

    /** @brief Stage in the convert pipeline */
    enum class EXPORT_API HistogramStage
    {
            noHistogram = 0,

            rawHistogram,

            // Before and after operation
            //beforeBayerBlackSubtract,
            afterBayerBlackSubtract,

            //beforeBayerQuickDemosaick,
            afterBayerQuickDemosaick,

            //beforeClarity2012,
            afterClarity2012,

            //beforeCreateBaseProxy,
            afterCreateBaseProxy,

            //beforeFixChromaticAberration,
            afterFixChromaticAberration,

            //beforeLensLightFalloff,
            afterLensLightFalloff,

            //beforeLocalToneMappingV8,
            afterLocalToneMappingV8,

            //beforeNoiseReduction,
            afterNoiseReduction,

            //beforeSharpening,
            afterSharpening,

            //beforeTetrahedral,
            afterTetrahedral,

            //beforeColorManagement,
            afterColorManagement,

            // Color management sub tasks
            beforeFilmCurve,
            beforeSaturation,
            beforeGamma,
            beforeContrast,
            beforeBrightness,
            beforeLevels,
            beforeClamp,
            beforeCurves,
            beforeColorBalance,
    };

    class EXPORT_API Histogram
    {
        public:
            Histogram();
            Histogram(HistogramType type, int binSize, int numChannel);

            /** @brief Histogram divides data range to series of intervals. Each interval is a bin.*/
            int GetBinSize(){return mBinSize;}

            /** @brief Return histogram data type. */
            HistogramType GetType(){return mType;}

            /** @brief Histogram counts value in a bin. Cumulative histogram counts value in a bin plus all smaller value.*/
            bool IsCumulative(){return mIsCumulative;}

            /** @brief Data type of the channel is defined by HistogramType. */
            const int* GetChannel(int channel);

            const int GetNumChannel(){return mNumChannel;}

            /** @brief Array of integer representing histogram data. */
            int* GetData(){return mData.data(); }

            /** @brief Size in bytes of the histogram data */
            int GetDataSizeBytes(){return mData.size() * sizeof(int); }

        protected:
            int mBinSize;
            HistogramType mType;
            int mNumChannel;
            bool mIsCumulative;
            std::vector<int> mData;
    };
}
}
